/******************************************************************************************************************************************
 *
 * Filename    : main.c
 * Project     : MemAlloc
 * Description : This is an example implementation of the C memory functions malloc(), free(), calloc() and realloc().
 *               It should serve as an tutorial for learning - it's not an efficient, optimized implementation.
 *
 *               See the corresponding article on my homepage for explanation of the algorithms.
 *
 * Sunshine, October 2k19
 * www.sunshine2k.de | www.bastian-molkenthin.de
 *
 * This file is licensed under MIT licence. For further details, see http://www.sunshine2k.de/license.html.
 *
 *****************************************************************************************************************************************/

/******************************************************************************************************************************************
 * INCLUDES
 *****************************************************************************************************************************************/
#include <stdio.h>
#include "mem1.h"
#include "mem2.h"
#include "mem1test.h"
#include "mem2test.h"
#include "testutil.h"

/******************************************************************************************************************************************
 * VARIABLES
 *****************************************************************************************************************************************/

/******************************************************************************************************************************************
 * FUNCTIONS
 *****************************************************************************************************************************************/

/******************************************************************************************************************************************
 * Name: main
 *****************************************************************************************************************************************/
int main()
{
  printf("Sample implementation of C memory functions including test cases\n");
  printf("Sunshine, October 2k19\n");
  printf("www.sunshine2k.de | www.bastian-molkenthin.de\n\n");
#if defined ( MEM1_TEST_MODE ) || defined ( MEM2_TEST_MODE )
  test_init();

# if defined ( MEM1_TEST_MODE )
  mem1test_FullHeapAlloc();
  mem1test_FullHeapAllocInSecondRequest();
  mem1test_FullHeapAllocInSecondRequest_FreeReversedOrder();
  mem1test_AllocRequestSizeTooLarge();
  mem1test_FreeBasic();
  mem1test_FreeLatestToFirst();
  mem1test_FreeFirstToLatest();
  mem1test_FreeMiddlePrevious();
  mem1test_FreeMiddleNext();
  mem1test_FreeMiddleTwoPrevious();
  mem1test_FreeMiddleTwoNext();
  mem1test_FreeMiddleAllocateAgain();
  mem1test_ReallocLargerSize();
  mem1test_ReallocSmallerSize();
# endif

# if defined ( MEM2_TEST_MODE )
  mem2test_FullHeapAlloc();
  mem2test_FullHeapAllocInSecondRequest();
  mem2test_FullHeapAllocInSecondRequest_FreeReversedOrder();
  mem2test_AllocRequestSizeTooLarge();
  mem2test_FreeBasic();
  mem2test_FreeLatestToFirst();
  mem2test_FreeFirstToLatest();
  mem2test_FreeMiddlePrevious();
  mem2test_FreeMiddleNext();
  mem2test_FreeMiddleTwoPrevious();
  mem2test_FreeMiddleTwoNext();
  mem2test_ReallocLargerSize();
  mem2test_ReallocSmallerSize();
#endif

  printf("\n");
  if (test_get_test_failed() == 0)
  {
    printf("Overall verdict: PASSED\n\n");
  }
  else
  {
    printf("Overall verdict: FAILED\n\n");
  }
#endif

#if defined _DEBUG
  printf("Press return key to quit...\n");
  getchar();
#endif

  return 0;
}

