/******************************************************************************************************************************************
 *
 * Filename    : mem1.h
 * Project     : MemAlloc
 * Description : This is an example implementation of the C memory functions malloc(), free(), calloc() and realloc().
 *               It should serve as an tutorial for learning - it's not an efficient, optimized implementation.
 *
 *               See the corresponding article on my homepage for explanation of the algorithms.
 *
 *               Header file for implementation variant 1 of the C memory functions.
 *
 * Sunshine, October 2k19
 * www.sunshine2k.de | www.bastian-molkenthin.de
 *
 * This file is licensed under MIT licence. For further details, see http://www.sunshine2k.de/license.html.
 *
 *****************************************************************************************************************************************/
 
#pragma once

/******************************************************************************************************************************************
 * INCLUDES
 *****************************************************************************************************************************************/
#include <stdint.h>

/******************************************************************************************************************************************
 * DEFINES
 *****************************************************************************************************************************************/
#define MEM1_TEST_MODE

#if !defined (NULL_PTR)
# define NULL_PTR   ((void*)0)
#endif

/******************************************************************************************************************************************
 * FUNCTION DECLARATIONS
 *****************************************************************************************************************************************/
void      mem1_init();

void*     mem1_malloc(size_t size);
void      mem1_free(void* p);
void*     mem1_calloc(size_t num, size_t size);
void*     mem1_realloc (void* ptr, size_t size);

void*     mem1_memset(void* ptr, int value, size_t num);
void*     mem1_memcpy(void* dest, const void * source, size_t num);


#if defined ( MEM1_TEST_MODE )
void      mem1_printHeap();
void      mem1_printHeapIfFailed();
uint32_t  mem1_getSizeOfHeapCtrlNode();
uint32_t  mem1_getSizeOfTotalHeapMem();

uint8_t   mem1_getNumberOfNodes();
uint8_t   mem1_getHeapUseOfNode(uint8_t numOfNode);
uint32_t  mem1_getHeapSizeOfNode(uint8_t numOfNode);
uint8_t   mem1_isHeapMemorySizeValid(void);
#endif


