/******************************************************************************************************************************************
 *
 * Filename    : mem2.h
 * Project     : MemAlloc
 * Description : This is an example implementation of the C memory functions malloc(), free(), calloc() and realloc().
 *               It should serve as an tutorial for learning - it's not an efficient, optimized implementation.
 *
 *               See the corresponding article on my homepage for explanation of the algorithms.
 *
 *               Header file for implementation variant 2 of the C memory functions.
 *
 * Sunshine, October 2k19
 * www.sunshine2k.de | www.bastian-molkenthin.de
 *
 * This file is licensed under MIT licence. For further details, see http://www.sunshine2k.de/license.html.
 *
 *****************************************************************************************************************************************/
 
#pragma once

/******************************************************************************************************************************************
 * INCLUDES
 *****************************************************************************************************************************************/
#include <stdint.h>

/******************************************************************************************************************************************
 * DEFINES
 *****************************************************************************************************************************************/
#define MEM2_TEST_MODE

#if !defined (NULL_PTR)
# define NULL_PTR   ((void*)0)
#endif

/******************************************************************************************************************************************
 * FUNCTION DECLARATIONS
 *****************************************************************************************************************************************/
void      mem2_init();

void*     mem2_malloc(size_t size);
void      mem2_free(void* p);
void*     mem2_calloc(size_t num, size_t size);
void*     mem2_realloc (void* ptr, size_t size);

void*     mem2_memset(void* ptr, int value, size_t num);
void*     mem2_memcpy(void* dest, const void * source, size_t num);


#if defined ( MEM2_TEST_MODE )
void      mem2_printHeapheader();
void      mem2_printHeap();
void      mem2_printHeapIfFailed();
uint32_t  mem2_getSizeOfHeapCtrlNode();
uint32_t  mem2_getSizeOfTotalHeapMem();
void*     mem2_getHeapStart();

uint8_t   mem2_getNumberOfNodes();
uint32_t  mem2_getHeapSizeOfNode(uint8_t numOfNode);
uint8_t   mem2_isHeapMemorySizeValid(void);

uint32_t  mem2_getEmptyHeapSpace();
#endif



