/******************************************************************************************************************************************
 *
 * Filename    : testutil.c
 * Project     : MemAlloc
 * Description : This is an example implementation of the C memory functions malloc(), free(), calloc() and realloc().
 *               It should serve as an tutorial for learning - it's not an efficient, optimized implementation.
 *
 *               See the corresponding article on my homepage for explanation of the algorithms.
 *
 *               Contains test utility functions.
 *
 * Sunshine, October 2k19
 * www.sunshine2k.de | www.bastian-molkenthin.de
 *
 * This file is licensed under MIT licence. For further details, see http://www.sunshine2k.de/license.html.
 *
 *****************************************************************************************************************************************/
 
/******************************************************************************************************************************************
 * INCLUDES
 *****************************************************************************************************************************************/
#include <stdio.h>
#include "testutil.h"

/******************************************************************************************************************************************
 * VARIABLES
 *****************************************************************************************************************************************/
uint8_t testcaseFailed;
uint8_t testFailed;

/******************************************************************************************************************************************
 * FUNCTIONS
 *****************************************************************************************************************************************/

/**************************************************************************************************
 * test_init()
 * Inits the test run.
 **************************************************************************************************/
void test_init(void)
{
  testcaseFailed = 0;
  testFailed = 0;
}

/**************************************************************************************************
 * test_assert_not_null_ptr()
 * Asserts that given pointer is not NULL. Testcase fails if pointer is NULL.
 **************************************************************************************************/
void test_assert_not_null_ptr(void* p)
{
  if (!p)
  {
    printf("Fail: NULL pointer!\n");
    testcaseFailed = 1;
  }
}

/**************************************************************************************************
 * test_assert_null_ptr()
 * Asserts that given pointer is NULL. Testcase fails if pointer is not NULL.
 **************************************************************************************************/
void test_assert_null_ptr(void* p)
{
  if (p)
  {
    printf("Fail: Memory allocated, but NULL pointer expected!\n");
    testcaseFailed = 1;
  }
}

/**************************************************************************************************
 * test_assert_equal()
 * Asserts that the two given values are equal. Testcase fails if the values are not equal.
 **************************************************************************************************/
void test_assert_equal(uint32_t isValue, uint32_t expValue)
{
  if (isValue == expValue)
  {
    printf("X");
  }
  else
  {
    printf("F");
    testcaseFailed = 1;
  }
}

/**************************************************************************************************
 * test_testcase_start()
 * Starts a new test case.
 **************************************************************************************************/
void test_testcase_start(char* testcasetitle)
{
  testcaseFailed = 0;
  printf(":: Test case start: %s\n", testcasetitle);
}

/**************************************************************************************************
 * test_testcase_end()
 * Ends a test case.
 **************************************************************************************************/
void test_testcase_end()
{
  if (testcaseFailed)
  {
    testFailed++;
  }
  printf("\n:: Test case end \n");
}

/**************************************************************************************************
 * test_get_testcase_failed()
 * Returns verdict of current test case. Returns 0 if test case is currently passed, otherwise 1.
 **************************************************************************************************/
uint8_t test_get_testcase_failed(void)
{
  return testcaseFailed;
}

/**************************************************************************************************
 * test_get_testcase_failed()
 * Returns verdict of current test run. Returns 0 if test run is currently passed, otherwise 1.
 **************************************************************************************************/
uint8_t test_get_test_failed(void)
{
  return testFailed;
}
